﻿using Microsoft.EntityFrameworkCore;
using SoftwareInstallationContracts.BindingModels;
using SoftwareInstallationContracts.SearchModels;
using SoftwareInstallationContracts.StoragesContracts;
using SoftwareInstallationContracts.ViewModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SoftwareInstallationDatabaseImplement
{
    public class OrderStorage : IOrderStorage
    {
        public List<OrderViewModel> GetFullList()
        {
            using var context = new SoftwareInstallationDatabase();
            return context.Orders
                    .Include(x => x.Package)
                    .Select(x => x.GetViewModel)
                    .ToList();
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (!model.Id.HasValue && !model.DateFrom.HasValue)
            {
                return new();
            }
            using var context = new SoftwareInstallationDatabase();
            if (model.DateFrom.HasValue)
            {
                return context.Orders
                    .Include(x => x.Package)
                    .Where(x => x.DateCreate >= model.DateFrom && x.DateCreate <= model.DateTo)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            return context.Orders
                    .Include(x => x.Package)
                    .Where(x => x.Id == model.Id)
                    .Select(x => x.GetViewModel)
                    .ToList();
        }

        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            using var context = new SoftwareInstallationDatabase();
            return context.Orders
                    .Include(x => x.Package)
                    .FirstOrDefault(x => x.Id == model.Id)
                    ?.GetViewModel;
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            using var context = new SoftwareInstallationDatabase();
            context.Orders.Add(newOrder);
            context.SaveChanges();
            return context.Orders
                      .Include(x => x.Package)
                      .FirstOrDefault(x => x.Id == newOrder.Id)
                      ?.GetViewModel;
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new SoftwareInstallationDatabase();
            var order = context.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }
            order.Update(model);
            context.SaveChanges();
            return context.Orders
                         .Include(x => x.Package)
                         .FirstOrDefault(x => x.Id == model.Id)
                         ?.GetViewModel;
        }

        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new SoftwareInstallationDatabase();
            var element = context.Orders.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                var deletedElement = context.Orders
                                           .Include(x => x.Package)
                                           .FirstOrDefault(x => x.Id == model.Id)
                                           ?.GetViewModel;
                context.Orders.Remove(element);
                context.SaveChanges();
                return deletedElement;
            }
            return null;
        }
    }
}
